import { ParseError } from './state.js'
import { clone, fail, succeed } from './fn.js'
import { anyChar } from './byte.js'
import { ok } from 'kojima'
import { chain, curry, pipe } from 'izuna'

/** @import { Result } from '../vendor/kojima/src/index.js' */
/** @import { ParserState } from './state.js' */

export const maybe = curry(
  /**
    * @param {(...args: any[]) => Result<ParserState, ParseError>} parser
    * @param {ParserState} state
    */
  (parser, state) => {
    const result = parser(clone(state))
    return result.isOk() ? result : succeed([], state)
  })

export const not = curry((parser, state) => {
  const result = parser(clone(state))

  if (result.isOk()) {
    return fail(`'not' parser failed`, state)
  } else {
    return succeed([], state)
  }
})

export const until = curry((parser, state) => {
  let result = ok(state)

  while (result.isOk()) {
    result = result.chain(pipe(clone, parser))
    if (result.isOk()) {
      break
    } else {
      result = anyChar(state)
    }
  }

  return result
})

export const verify = curry((parser, predicate, state) => {
  const result = chain(parser, clone(state))
  return chain(x => (
    predicate(x) ? result : fail('verification failed', state)
  ), result)
})

export const skip = curry((parser, state) => {
  return chain(_ => succeed([], state), parser(state))
})

