import { err, ok } from 'kojima'
import { concat, curry } from 'izuna'
import { ParseError, state } from './state.js'
import { IndexableIterator } from './iter.js'

/** @import { ParserState } from './state.js' */


export const iter = value => new IndexableIterator(value)

/**
 * @param {ParserState} state
 */
export const clone = (state) => state.clone()

/**
 * @template T
 * @param {T | T[]} v
 * @param {ParserState} state
 */
export const succeed = (v, [x, y]) => {
  return ok(state(y, concat(v, x)))
}

/**
 * @param {string} msg
 * @param {ParserState} state
 * @param {Error} [e]
 */
export const fail = (msg, state, e = undefined) => err(new ParseError(msg, state, e))

/**
 * @template T
 * @param {number & keyof T} n
 * @param {T[]} iter
 */
export const nth = (n, iter) => iter[n]

/**
 * @param {ParserState} state
 */
export const next = state => state.next().value

/**
 * @template T
 * @param {T[]} a
 * @param {T[]} b
 */
export const diff = (a, b) => b.slice(-Math.max(0, b.length - a.length))

export const join = curry(
  /**
    * @param {string} delim
    * @param {string[]} val
    */
  (delim, val) => val.join(delim)
)

export const mapStr = curry(
  /**
    * @param {(...args: any[]) => any} fn
    * @param {string} str
    */
  (fn, str) => Array.from(str).flatMap(x => {
    return fn(x)
  })
)


